/*HEADER**********************************************************************
*
* Copyright 2008, 2014 Freescale Semiconductor, Inc.
* Copyright 1989-2008 ARC International
*
* This software is owned or controlled by Freescale Semiconductor.
* Use of this software is governed by the Freescale MQX RTOS License
* distributed with this Material.
* See the MQX_RTOS_LICENSE file distributed for more details.
*
* Brief License Summary:
* This software is provided in source form for you to use free of charge,
* but it is not open source software. You are allowed to use this software
* but you cannot redistribute it or derivative works of it in source form.
* The software may be used only in connection with a product containing
* a Freescale microprocessor, microcontroller, or digital signal processor.
* See license agreement file for full license terms including other
* restrictions.
*****************************************************************************
*
* Comments:
*
*   This file contains the source for the hello example program.
*
*END************************************************************************/

#include <shell.h>
#include <sh_rtcs.h>
#include <log.h>
#include <klog.h>
#if PLATFORM_SDK_ENABLED
  #include <stdio.h>
  #include <fsl_os_abstraction.h>
  #include <fsl_gpio_driver.h>
  #include <board.h>
  #include "app_rtos.h"
  #include "fsl_rnga_driver.h"

#else
  #include <mqx.h>
  #include <bsp.h>
#endif

#include "main.h"
#include <wolfssl/wolfcrypt/settings.h>

#if !PLATFORM_SDK_ENABLED
  #if !BSPCFG_ENABLE_IO_SUBSYSTEM
  #error This application requires BSPCFG_ENABLE_IO_SUBSYSTEM defined non-zero in mqx_sdk_config.h. Please recompile BSP with this option.
  #endif


  #ifndef BSP_DEFAULT_IO_CHANNEL_DEFINED
  #error This application requires BSP_DEFAULT_IO_CHANNEL to be not NULL. Please set corresponding BSPCFG_ENABLE_TTYx to non-zero in mqx_sdk_config.h and recompile BSP with this option.
  #endif
#endif

const SHELL_COMMAND_STRUCT Shell_commands[] = {
   { "help",      Shell_help },
   { "?",         Shell_command_list },

   /* RTCS command.*/
#if RTCSCFG_ENABLE_ICMP
    { "ping",      Shell_ping },
#endif

   { "ipconfig",  Shell_ipconfig },
   { "email",     Shell_smtp },
   { "sec_email", Shell_smtp_ssl },
   { NULL,        NULL }
};

#if !PLATFORM_SDK_ENABLED
TASK_TEMPLATE_STRUCT MQX_template_list[] =
{
/*  Task number, Entry point, Stack, Pri, String, Auto? */
   {MAIN_TASK,   main_task,   14000,  9 /* //DM 9*/,   "main", MQX_AUTO_START_TASK},
   {0,           0,           0,     0,   0,      0}
};

typedef uint32_t task_param_t;
#endif


/*TASK*-----------------------------------------------------------------
*
* Task Name    : main_task
* Comments     : The benchmark shell.
*
*END*************************************************************************/
void main_task ( task_param_t init_data )
{
    uint32_t     error;
    rnga_user_config_t rngaConfig;
    DATE_STRUCT date;
    TIME_STRUCT time;

    date.YEAR = 2016;
    date.MONTH = 1;
    date.DAY = 25;
    date.HOUR = 12;
    date.MINUTE = 9;
    date.SECOND = 0;
    date.MILLISEC = 0;

    _time_from_date(&date, &time);
    _time_set(&time);

    //init RNGA, necessary for SSL
    rngaConfig.isIntMasked         = true;
    rngaConfig.highAssuranceEnable = true;
    NVIC_SetPriority(g_rngaIrqId[RNGA_INSTANCE], 6U);
    RNGA_DRV_Init(RNGA_INSTANCE, &rngaConfig);

   /* runtime RTCS configuration for devices with small RAM, for others the default BSP setting is used */
//   _RTCSPCB_init = 4;
//   _RTCSPCB_grow = 2;
//   _RTCSPCB_max = 20;
//   _RTCS_msgpool_init = 4;
//   _RTCS_msgpool_grow = 2;
//   _RTCS_msgpool_max  = 20;
//   _RTCS_socket_part_init = 4;
//   _RTCS_socket_part_grow = 2;
//   _RTCS_socket_part_max  = 20;
//
//    _RTCSTASK_stacksize = 3000;


    error = RTCS_create();
    if (error != RTCS_OK)
    {
        printf("\nRTCS failed to initialize, error = %X", error);
        _task_block();
    }
    /* Enable IP forwarding */
    _IP_forward = TRUE;

	for (;;)
   {
      /* Run the shell */
      Shell(Shell_commands, NULL);
      printf("Shell exited, restarting...\r\n");
   }
}

/* EOF */
