/* settings.h
 *
 * Copyright (C) 2006-2014 wolfSSL Inc.  All rights reserved.
 *
 * This file is part of WOLFSSL.
 *
 * Contact licensing@wolfssl.com with any questions or comments.
 *
 * http://www.wolfssl.com
 */

/**********************************************************************
* Settigs of WOLFSSL library, for RTCS/MQX.
***********************************************************************/

#ifndef __WOLFSSL_SETTINGS_H__
#define __WOLFSSL_SETTINGS_H__

#include <wolfssl/wolfcrypt/visibility.h>

#define SIZEOF_LONG_LONG 8
#define NO_WRITEV
#define NO_DEV_RANDOM
#define NO_RABBIT
#define NO_WOLFSSL_DIR
/*Setting for PIT timer*/
#define  PIT_INSTANCE   0
#define  PIT_CHANNEL    0
/* Enable settings for low-memory platforms, MCUs.*/
//#define RSA_LOW_MEM
#define TFM_TIMING_RESISTANT
#define ECC_TIMING_RESISTANT

#if defined(__GNUC__)   /* CW GCC workaround. gmtime() is not availble*/
    #define USER_TIME
#endif

#define HAVE_ECC
#define HAVE_AESCCM
#define HAVE_AESGCM
#define WOLFSSL_AES_COUNTER (1)
#define WOLFSSL_AES_DIRECT (1)

#ifdef __cplusplus
    extern "C" {
#endif

#include "fsl_device_registers.h"

/* Enable TRNG use in library if there is TRNG on chip. */ 
#if (FSL_FEATURE_SOC_TRNG_COUNT > 0)
    #include "fsl_trng_driver.h"
    #define FREESCALE_TRNG
    #define TRNG_INSTANCE (0)
#elif (FSL_FEATURE_SOC_RNG_COUNT > 0)
    #include "fsl_rnga_driver.h"
    #define FREESCALE_RNGA
    #define RNGA_INSTANCE (0)
#endif

/* Enable LTC use in library if there is LTC on chip. */
#if (FSL_FEATURE_SOC_LTC_COUNT > 0)
    #include "fsl_ltc_driver.h"
    #define FREESCALE_LTC (1)
    #define LTC_INSTANCE (0)
    #if (FSL_FEATURE_LTC_HAS_SHA > 0)
        #define FREESCALE_LTC_SHA (1)
        #define FREESCALE_MMCAU_SHA (0)
    #elif (FSL_FEATURE_SOC_MMCAU_COUNT > 0)
        #define FREESCALE_MMCAU_SHA (1)
    #endif
    #if (FSL_FEATURE_LTC_HAS_PKHA > 0)
        #define FREESCALE_LTC_PKHA (1)
    #endif
    /* ECC-256, ECC-224 and ECC-192 are supported by LTC PKHA acceleration */
    #ifdef HAVE_ECC
        #ifndef ECC_TIMING_RESISTANT
            #define ECC_TIMING_RESISTANT (1)
        #endif
        #define HAVE_ECC192 (1)
        #define HAVE_ECC224 (1)
        #define LTC_MAX_ECC (256)
        #define LTC_MAX_INT (256)
    #endif
#endif

/* Enable MMCAU library. */
#if (FSL_FEATURE_SOC_MMCAU_COUNT > 0)
    #define FREESCALE_MMCAU (1)
    /* FreeScale MMCAU hardware crypto has 4 byte alignment */
    #define WOLFSSL_MMCAU_ALIGNMENT 4
    
    #if !(FREESCALE_LTC_SHA)
        #define FREESCALE_MMCAU_SHA (1)
    #else
        #define FREESCALE_MMCAU_SHA (0)
    #endif

    #if !(FREESCALE_LTC)
        #undef WOLFSSL_AES_COUNTER
    #endif
#endif

#if defined(FREESCALE_KSDK_BM)
#define WOLFSSL_USER_IO
#define SINGLE_THREADED
#define NO_FILESYSTEM
#define USE_WOLFSSL_MEMORY
#endif /* FREESCALE_KSDK_BM */

/* if using hardware crypto and have alignment requirements, specify the
   requirement here.  The record header of SSL/TLS will prvent easy alignment.
   This hint tries to help as much as possible.  */
#ifndef WOLFSSL_GENERAL_ALIGNMENT
    #ifdef FREESCALE_MMCAU
        #define WOLFSSL_GENERAL_ALIGNMENT  WOLFSSL_MMCAU_ALIGNMENT
    #else
        #define WOLFSSL_GENERAL_ALIGNMENT  0
    #endif
#endif

/* Definitions for FreeRTOS */
#ifdef FREESCALE_FREE_RTOS
    #define WOLFSSL_USER_IO
    #define NO_FILESYSTEM
    #define NO_MAIN_DRIVER
    #define XMALLOC(s, h, t)  OSA_MemAlloc(s);(void)h;(void)t;
    #define XFREE(p, h, t)    {void* xp = (p); if((xp)) OSA_MemFree((xp));}
    #ifdef FREESCALE_KSDK_BM
      #error Baremetal and FreeRTOS cannot be both enabled at the same time!
    #endif
    #ifndef SINGLE_THREADED
        #include "FreeRTOS.h"
        #include "semphr.h"
    #endif
#endif

#if defined(FREESCALE_MQX)
#define errno   RTCS_errno
#include <stdio.h>
#include <mqx.h>
#include <nio.h>
#include <mutex.h>
#define MQX_FILE_PTR FILE *
#define IO_SEEK_SET SEEK_SET
#define IO_SEEK_END SEEK_END

#define EAGAIN  NIO_EAGAIN
#define EINTR   NIO_EINTR
#define EPIPE   NIO_EPIPE

/* Memory functions.*/
#define XMALLOC(s, h, t)        (void *)_mem_alloc_system((s))
#define XFREE(p, h, t)          {void* xp = (p); if ((xp)) _mem_free((xp));}
#define XREALLOC(p, n, h, t)    _mem_realloc((p), (n)) /* Since MQX 4.1.2 */
//#define WOLFSSL_SMALL_STACK
#endif /* FREESCALE_MQX */

/* Place any other flags here */
#define DEBUG_WOLFSSL 	1

#ifdef __cplusplus
    }   /* extern "C" */
#endif

#endif /* __WOLFSSL_SETTINGS_H__ */

